/************************************************************************/
/*																		*/
/*	Dspi.c	--	Interface Declarations for PIC32 SPI PORTS				*/
/*																		*/
/************************************************************************/
/*	Author:		Oliver Jones											*/
/*	Copyright 2011, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*																		*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	07/07/2011(OliverJ): created										*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include <p32xxxx.h>
#include "stdtypes.h"
#include "Dspi.h"
#include "Dport.h"
#include "Dclock.h"
#include "Dmgr.h"

/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Forward Declarations							*/
/* ------------------------------------------------------------ */

BYTE SpiPutByte(spiHIF * hif, BYTE bData);

/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */
/***	BOOL DspiGetHandle(spiHIF * phif, unsigned int prtReq)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		prtReq			- SPI port # request
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Initializes SPI port handle
**
*/

BOOL DspiGetHandle(spiHIF * hif, unsigned int prtReq)
{
	//Set up spi handle for requested port
	switch(prtReq) {
		case	spi0:
			((spiHIF*)hif)->type = modSpiBitBang;
			hif->spiCon.spiBitBang.fEnabled = fTrue;
			return fTrue;

		case	spi1:
			#if defined (_SPI1)
				hif->type = modSpiHardware;
				hif->spiCon.spiHardware.portNumber = prtReq;
				hif->spiCon.spiHardware.pConSet = (unsigned int*)&SPI1CONSET;
				hif->spiCon.spiHardware.pConClr = (unsigned int*)&SPI1CONCLR;
				hif->spiCon.spiHardware.pStat = (unsigned int*)&SPI1STAT;
				hif->spiCon.spiHardware.pStatSet = (unsigned int*)&SPI1STATSET;
				hif->spiCon.spiHardware.pStatClr = (unsigned int*)&SPI1STATCLR;
				hif->spiCon.spiHardware.pBaud = (unsigned int*)&SPI1BRG;
				hif->spiCon.spiHardware.pBuffer = (unsigned int*)&SPI1BUF;
				break;
			#else
				setLastError(1);
				return fFalse;
			#endif

		case	spi2:
			#if defined (_SPI2)
				hif->type = modSpiHardware;
				hif->spiCon.spiHardware.portNumber = prtReq;
				hif->spiCon.spiHardware.pConSet = (unsigned int*)&SPI2CONSET;
				hif->spiCon.spiHardware.pConClr = (unsigned int*)&SPI2CONCLR;
				hif->spiCon.spiHardware.pStat = (unsigned int*)&SPI2STAT;
				hif->spiCon.spiHardware.pStatSet = (unsigned int*)&SPI2STATSET;
				hif->spiCon.spiHardware.pStatClr = (unsigned int*)&SPI2STATCLR;
				hif->spiCon.spiHardware.pBaud = (unsigned int*)&SPI2BRG;
				hif->spiCon.spiHardware.pBuffer = (unsigned int*)&SPI2BUF;
				break;
			#else
				setLastError(1);
				return fFalse;
			#endif

		default:
			setLastError(2);
			return fFalse;
	}

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	void DspiEnable(spiHIF * hif)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Turns on SPI device and sets up some default registers
**		values
**
*/

void DspiEnable(spiHIF * hif)
{
	// Turn on SPI device
	//*hif->spiCon.spiHardware.pConSet = (1 << bnOn)  | (1<<bnSmp)| (1 << bnMsten);
	*hif->spiCon.spiHardware.pConSet = (1 << bnOn)  | (1 << bnMsten);
	
}

/* ------------------------------------------------------------ */
/***	void DspiDisable(spiHIF * hif)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Turns off SPI device
**
*/

void DspiDisable(spiHIF * hif)
{	
	// Turn off SPI device
	*hif->spiCon.spiHardware.pConClr = (1 << bnOn);
}

/* ------------------------------------------------------------ */
/***	BOOL DspiSetPinSS(spiHIF * hif, char chPort, unsigned int pin)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		chPort			- Letter of Requested port
**		pin				- Pin value of requested port
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Sets the SS with the desired port and pin value
**
*/

BOOL DspiSetPinSS(spiHIF * hif, char chPort, unsigned int pin)
{
	// Get a handle to the requested port
	if(DportGetHandle(&hif->portSS,chPort) == fFalse) {
		return fFalse;
	}

	// Check if pin is out of bounds
	if(pin >= 32) {
		setLastError(2);
		return fFalse;
	}

	// Set direction of pin		
	if(DportSetPinDir(&hif->portSS, pin, fTrue) == fFalse) {
		return fFalse;
	}

	hif->pinSS = pin;

	return fTrue;	
}

/* ------------------------------------------------------------ */
/***	BOOL DspiSetPinClk(spiHIF * hif, char chPort, unsigned int pin)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		chPort			- Letter of Requested port
**		pin				- Pin value of requested port
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Sets the Clk with the desired port and pin value
**
*/

BOOL DspiSetPinClk(spiHIF * hif, char chPort, unsigned int pin)
{
	// Ensure that handle is to a bit bang SPI port
	if(hif->type == modSpiHardware) {
		setLastError(3);
		return fFalse;
	}

	// Get a handle to the requested port
	if(DportGetHandle(&hif->spiCon.spiBitBang.portClk,chPort) == fFalse) {
		return fFalse;
	}

	// Check if pin is out of bounds
	if(pin >= 32) {
		setLastError(2);
		return fFalse;
	}

	// Set direction of pin
	if(DportSetPinDir(&hif->spiCon.spiBitBang.portClk, pin, fTrue) == fFalse) {
		return fFalse;
	}

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DspiSetPinMosi(spiHIF * hif, char chPort, unsigned int pin)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		chPort			- Letter of Requested port
**		pin				- Pin value of requested port
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Sets the Mosi with the desired port and pin value
**
*/

BOOL DspiSetPinMosi(spiHIF * hif, char chPort, unsigned int pin)
{
	// Ensure that handle is to a bit bang SPI port
	if(hif->type == modSpiHardware) {
		setLastError(3);
		return fFalse;
	}

	// Get a handle to the requested port
	if(DportGetHandle(&hif->spiCon.spiBitBang.portMosi,chPort) == fFalse) {
		return fFalse;
	}

	// Check if pin is out of bounds
	if(pin >= 32) {
		setLastError(2);
		return fFalse;
	}

	// Set direction of pin
	if(DportSetPinDir(&hif->spiCon.spiBitBang.portMosi, pin, fTrue) == fFalse) {
		return fFalse;
	}

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DspiSetPinMiso(spiHIF * hif, char chPort, unsigned int pin)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		chPort			- Letter of Requested port
**		pin				- Pin value of requested port
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Sets the Mosi with the desired port and pin value
**
*/

BOOL DspiSetPinMiso(spiHIF * hif, char chPort, unsigned int pin)
{
	// Ensure that handle is to a bit bang SPI port
	if(hif->type == modSpiHardware) {
		setLastError(3);
		return fFalse;
	}

	// Get a handle to the requested port
	if(DportGetHandle(&hif->spiCon.spiBitBang.portMiso,chPort) == fFalse) {
		return fFalse;
	}

	// Check if pin is out of bounds
	if(pin >= 32) {
		setLastError(2);
		return fFalse;
	}

	// Set direction of pin	
	if(DportSetPinDir(&hif->spiCon.spiBitBang.portMiso, pin, fFalse) == fFalse) {
		return fFalse;
	}
			
	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DspiSetSelect(spiHIF * hif, BOOL fSel)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		fSel			- Sets output of pin
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Sets the output of the Set Select pin based on fSel
**		if fSel is true the Set Select pin will be logic
**		high (1) otherwise it will be logic low (0)
**
*/

BOOL DspiSetSelect(spiHIF * hif, BOOL fSel)
{
	return DportSetPinOutput(&hif->portSS, hif->pinSS, fSel);
}

/* ------------------------------------------------------------ */
/***	void DspiSetSpiMode(spiHIF * hif, DWORD idMod, BOOL fShRight)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		idMod			- Value to set spi mode	
**
**	Return Value:
**		none
**
**	Errors:
**		TBD
**
**	Description:
**		Sets Spi controller to one of the 4 availibe modes
**
*/

void DspiSetSpiMode(spiHIF * hif, DWORD idMod)
{
	// Check whether handle points to a hardware spi port
	// or a bit bang spi port
	if(hif->type == modSpiHardware) {
		// Bit mask out any erronius bits
		idMod = idMod & bmSpiMod;
		// Set necessary bits
		*hif->spiCon.spiHardware.pConSet = idMod;
		// Xor mode
		idMod = idMod ^ bmSpiMod;
		// Clear necessary bits
		*hif->spiCon.spiHardware.pConClr = idMod;		
	}
	else {
		hif->spiCon.spiBitBang.mode = idMod;
	}
}

/* ------------------------------------------------------------ */
/***	void DspiSetSpeed(spiHIF * hif, DWORD frqReq, DWORD * pfrqSet)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		frqReq			- Value to set spi speed
**
**	Return Value:
**		none
**
**	Errors:
**		TBD
**
**	Description:
**		Sets Spi controller speed to specified value
**
*/

void DspiSetSpeed(spiHIF * hif, DWORD frqReq)
{
	if(hif->type == modSpiHardware) {
		int pbClock = DclockGetPbClock();
		*hif->spiCon.spiHardware.pBaud = ( pbClock / (frqReq * 2) ) - 1;
	}
}

/* ------------------------------------------------------------ */
/***	BOOL DspiPutByte(spiHIF * hif, BOOL fSelStart, BOOL fSelEnd, BYTE bSnd,BYTE * pbRcv)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		fSelStart		- State of SS signal before byte is shifted to slave
**		fSelEnd			- State of SS signal after byte is shifted to slave
**		bSnd			- byte to shift in to the slave
**		pbRcv			- pointer to the recieve byte shifted out of slave. If set to null no byte is returned
**
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		This funtion sends a single byte to the slave device on the slave.
**		it simutaneously reads a byte from the slave and returns it in pbRcv.
**		If a NULL value is specified for pbRcv, then no byte will be returned
**
*/

//!!!!!!!!!!!!!! Modify so that fSelStar is no longer a bool but BYTE
// This for overloading in c++ wrapper
BOOL DspiPutByte(spiHIF * hif, BOOL fSelStart, BOOL fSelEnd, BYTE bSnd,BYTE * pbRcv)
{
	if(hif->type == modSpiHardware) {
		DportSetPinOutput(&hif->portSS, hif->pinSS, fSelStart);

		if(pbRcv == NULL) {
			SpiPutByte(hif, bSnd);
		}
		else {
			*pbRcv = SpiPutByte(hif, bSnd);
		}

		DportSetPinOutput(&hif->portSS, hif->pinSS, fSelEnd);
	}

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DspiPut(spiHIF * hif, BOOL fSelStart, BOOL fSelEnd, BYTE * rgbSnd,BYTE * pbRcv, DWORD cbSend)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		fSelStart		- State of SS signal before byte is shifted to slave
**		fSelEnd			- State of SS signal after byte is shifted to slave
**		rgbbSnd			- pointer to bytes to shift in to the slave
**		rpbRcv			- pointer to the recieve bytes shifted out of slave. If set to null no byte is returned
**		cbSnd			- number of bytes to put
**
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		This funtion sends an array of bytes to the slave device on the slave.
**		it simutaneously reads a byte from the slave and returns it in pbRcv.
**		If a NULL value is specified for pbRcv, then no byte will be returned
**
*/
		
BOOL DspiPut(spiHIF * hif, BOOL fSelStart, BOOL fSelEnd, BYTE * rgbSnd,BYTE * pbRcv, DWORD cbSend)
{
	if(hif->type == modSpiHardware) {
		int i;
		DportSetPinOutput(&hif->portSS, hif->pinSS, fSelStart);
		for(i=0; i < cbSend; i++) {
			if(pbRcv == NULL) {
				SpiPutByte(hif, *rgbSnd++);
			}
			else {
				*pbRcv++ = SpiPutByte(hif, *rgbSnd++);
			}
		}
		DportSetPinOutput(&hif->portSS, hif->pinSS, fSelEnd);
	}

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DspiGet(spiHIF * hif, BOOL fSelStart, BOOL fSelEnd, BYTE bFill,BYTE * pbRcv, DWORD cbRcv)
**
**	Parameters:
**		hif				- Pointer to spi port handle
**		fSelStart		- State of SS signal before byte is shifted from slave
**		fSelEnd			- State of SS signal after byte is shifted from slave
**		pbRcv			- pointer to bytes to shift in to the slave
**		cbRcv			- number of bytes to get
**
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		This funtion reads the specified number of bytes out of the slave on
**		an enabled port. The data read from the slave will be placed in the
**		buffer specified by rgbRcv.
**
*/

BOOL DspiGet(spiHIF * hif, BOOL fSelStart, BOOL fSelEnd, BYTE * pbRcv, DWORD cbRcv)
{
	if(hif->type == modSpiHardware) {
		int i;
		DportSetPinOutput(&hif->portSS, hif->pinSS, fSelStart);
		for(i=0; i < cbRcv; i++) {
			*pbRcv++ = SpiPutByte(hif, NULL);
		}
		DportSetPinOutput(&hif->portSS, hif->pinSS, fSelEnd);
	}

	return fTrue;
}

BYTE SpiPutByte(spiHIF * hif, BYTE bData)
{
	BYTE bRx;
	//if Transmit Buffer Empty
	while( ((1 << bnSpitbe) & *(hif->spiCon.spiHardware.pStat)) == 0 );
	*hif->spiCon.spiHardware.pBuffer = bData;
	//Receive Buffer Full
	while( ((1 << bnSpirbf) & *(hif->spiCon.spiHardware.pStat)) == 0 );
	bRx = *hif->spiCon.spiHardware.pBuffer;
	return bRx;
}
