/************************************************************************/
/*																		*/
/*	PmodSF.c	--	Definition for PmodSF library 	    				*/
/*																		*/
/************************************************************************/
/*	Author:		Michelle Yu												*/
/*	Copyright 2011, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*		This file defines functions for PmodSF							*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	07/29/2011(MichelleY): created										*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include "Dspi.h"


/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------------- */
/**BOOL WriteEnable(spiHIF * spiH)
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function enables the device to write or erase
**
-----------------------------------------------------------------------*/
BOOL WriteEnable(spiHIF * spiH)
{
	BYTE sndByte[1]={0};

	//set command  0x06 for write enable 
	sndByte[0] = 0x06;

	//Send command
	DspiPut(spiH, fFalse, fTrue, sndByte, NULL, 1);

	return fTrue;	

}

/* ------------------------------------------------------------------- */
/**BYTE ReadSR(spiHIF * spiH)
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**
**	Return Value:
**		Returns status register
**
**	Errors:
**		TBD
**
**	Description:
**		This function returns the status register
**
-----------------------------------------------------------------------*/
BYTE ReadSR(spiHIF * spiH)
{
	BYTE sndByte[1]={0};
	int rcvByte[1]={0};

	//set command  0x05 for read status register 
	sndByte[0] = 0x05;

	//Send command
	DspiPut(spiH, fFalse, fFalse, sndByte, NULL, 1);

	//Get status register
	DspiGet(spiH, fFalse, fTrue, rcvByte, 1);

	return fTrue;	

}

/* ------------------------------------------------------------------- */
/**BOOL WaitForWrite(spiHIF * spiH)
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function waits for memory to be finish with a write or erase cycle	
**
-----------------------------------------------------------------------*/
BOOL WaitForWrite(spiHIF * spiH)
{
	BYTE sndByte[1]={0};
	int rcvByte[1]={0};

	//set command  0x05 for read status register 
	sndByte[0] = 0x05;

	//Send command
	DspiPut(spiH, fFalse, fFalse, sndByte, NULL, 1);

	//Get status register
	DspiGet(spiH, fFalse, fFalse, rcvByte, 1); 
	
	//0x1 means a write is in progess
	while(rcvByte[0] & 0x1)
	{
		//keep polling status register
		DspiGet(spiH, fFalse, fFalse, rcvByte, 1); 
	}

	//set ss high to deselect the device
	DspiSetSelect(spiH, fTrue);

	return fTrue;	

}
/* ------------------------------------------------------------------- */
/**int GetDeviceInfo(spiHIF * spiH, int* maxAddress)
**
**	Parameters:
**		spiH				- Pointer to spi port handle
**		maxAddress		- pointer to store maximum address for the device
**
**
**	Return Value:
**		Returns Chip Model 
				0x16  - M25P16
				0x128 - M25P128
				0 	  - if anything else
**
**	Errors:
**		TBD
**
**	Description:
**		This function informs user device information 
**
-----------------------------------------------------------------------*/
int GetDeviceInfo(spiHIF* spiH, int* maxAddress)
{
	BYTE sndByte[1];
	unsigned int rcvByte[3];
	
	//set command  0x9F for Read ID
	sndByte[0] = 0x9F;	

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);

	//Send command
	DspiPut(spiH, fFalse, fFalse, sndByte, NULL, 1);
	//Clear the receive byte
	rcvByte[0] = 0; 
	//Receive data
	DspiGet(spiH, fFalse, fTrue, rcvByte, 3); 

	if(rcvByte[0] == 0x152020)
	{
		*maxAddress = 0x1FFFFF;		
		return 0x16;
	}
	else if(rcvByte[0] == 0x182020)
	{
		*maxAddress = 0xFFFFFF;	
		return 0x128;	
	}
	else
	{
		return fFalse;
	}	

}

/* ------------------------------------------------------------------- */
/**BOOL NormalRead(spiHIF * spiH, BYTE* readData, int address, DWORD byteNum)
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**		address		- a 3 byte address to read from
**		readData	- Pointer to an array to store data read
**		byteNum		- Number of byte desire to read
**		
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function reads ByteNum of data from the address specified
**
-----------------------------------------------------------------------*/
BOOL NormalRead(spiHIF * spiH, BYTE* readData, int address, DWORD byteNum)
{
	BYTE sndByte[4]={0,0,0,0,0};
	
	//set command  0x03 for Read at normal speed 
	sndByte[0] = 0x03;
	//set 3 byte address MSB first 
	sndByte[1] = (address >> 16);
	sndByte[2] = (address >> 8);
	sndByte[3] =  address;	

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);
	
	//Send command
	DspiPut(spiH, fFalse, fFalse, sndByte, NULL, 4);

	//Receive data
	DspiGet(spiH, fFalse, fTrue, readData, byteNum); 

	return fTrue;	

}

/* ------------------------------------------------------------------- */
/**BOOL FastRead(spiHIF * spiH, BYTE* readData, int address, DWORD byteNum)
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**		address		- a 3 byte address to read from
**		rearData	- Pointer to an array to store data read
**		byteNum		- Number of byte desire to read
**		
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function reads ByteNum of data from the address specified at a higher speed
**
-----------------------------------------------------------------------*/
BOOL FastRead(spiHIF * spiH, BYTE* readData, int address, DWORD byteNum)
{
	BYTE sndByte[5]={0,0,0,0,0};
	
	//set command  0x0B for Read at fast speed 
	sndByte[0] = 0x0B;
	//set 3 byte address MSB first 
	sndByte[1] = (address >> 16);
	sndByte[2] = (address >> 8);
	sndByte[3] = address;
	sndByte[4] = 0x00;

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);

	//Send command
	DspiPut(spiH, fFalse, fFalse, sndByte, NULL, 5);

	//Receive data
	DspiGet(spiH, fFalse, fTrue, readData, byteNum); 

	return fTrue;	

}


/* ------------------------------------------------------------------- */
/**BOOL WritePage(spiHIF * spiH, BYTE* writeData, int address, DWORD byteNum)
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**		address		- a 3 byte address to write to 
**		writeData	- Pointer to an array of data to write
**		byteNum		- Number of byte desire to write
**		
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function writes ByteNum of data to the address specified 
**		Write cycle begins after the function set SS high
**
-----------------------------------------------------------------------*/
BOOL WritePage(spiHIF * spiH, BYTE* writeData, int address, DWORD byteNum)
{
	BYTE sndByte[4]={0,0,0,0};

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);

	//enable write
	WriteEnable(spiH);
	
	//set command  0x02 for Page Program 
	sndByte[0] = 0x02;
	//set 3 byte address MSB first 
	sndByte[1] = (address >> 16);
	sndByte[2] = (address >> 8);
	sndByte[3] = (address);


	//Send command
	DspiPut(spiH, fFalse, fFalse, sndByte, NULL, 4);
	DspiPut(spiH, fFalse, fTrue, writeData, NULL, byteNum);

	return fTrue;	

}

/* ------------------------------------------------------------------- */
/**BOOL SectorErase(spiHIF * spiH, int Address)
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**		address		- a 3 byte address of the sector to be erased
**		
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function erases the sector that the address is in
**		Erase cycle begins after the function set SS high
**
-----------------------------------------------------------------------*/
BOOL SectorErase(spiHIF * spiH, int address)
{
	BYTE sndByte[4]={0,0,0,0};

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);

	//enable write
	WriteEnable(spiH);
	
	//set command  0xD8 for sector erase 
	sndByte[0] = 0xD8;
	//set 3 byte address MSB first 
	sndByte[1] = (address >> 16);
	sndByte[2] = (address >> 8);
	sndByte[3] = (address);

	//Send command
	DspiPut(spiH, fFalse, fTrue, sndByte, NULL, 4);

	return fTrue;	

}

/* ------------------------------------------------------------------- */
/**BOOL BulkErase(spiHIF * spiH)
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**		
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function erases the entire memory
**		Erase cycle begins after the function set SS high
**
-----------------------------------------------------------------------*/
BOOL BulkErase(spiHIF * spiH)
{
	BYTE sndByte[1];

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);

	//enable write
	WriteEnable(spiH);
	
	//set command  0xC7 for bulk erase 
	sndByte[0] = 0xC7;

	//Send command
	DspiPut(spiH, fFalse, fTrue, sndByte, NULL, 1);

	return fTrue;	

}

/* ------------------------------------------------------------------- */
/**BOOL WriteProtect(spiHIF * spiH, BYTE areaNum) 
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**		areaNum		- sector area number to protect against write/erase 
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
//	This funciton protects certain sectors against write/erase. The corresponding 
    areaNum for each sector are shown below.
  	ChipModel can be find with GetDeviceInfo();

	ChipModel 0x16:
		0x00 none 
		0x04 Upper 32nd (Sector 31) 
		0x08 Upper sixteenth (two sectors: 30 and 31)
		0x0C Upper eighth (four sectors: 28 to 31) 
		0x10 Upper quarter (eight sectors: 24 to 31) 
		0x14 Upper half (sixteen sectors: 16 to 31) 
		0x18 All sectors (32 sectors: 0 to 31) 
		0x1C All sectors (32 sectors: 0 to 31) 
	ChipModel 0x128:
		0x00 none 
		0x04 Upper 64th (1 Sector, 2Mb) 
		0x08 Upper 32nd (2 Sectors, 4Mb) 
		0x0C Upper 16nd (4 Sectors, 8Mb) 
		0x10 Upper 8nd (8 Sectors, 16Mb) 
		0x14 Upper Quarter (16 Sectors, 32Mb) 
		0x18 Upper Half (32 Sectors, 64Mb) 
		0x1C All sectors (64 Sectors, 128Mb) 
**
-----------------------------------------------------------------------*/
BOOL WriteProtect(spiHIF * spiH, BYTE areaNum) 
{

	BYTE sndByte[2]={0};
	
	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);

	//enable write
	WriteEnable(spiH);
		
	//set command  0x01 to Write to Status Register
	sndByte[0] = 0x01;
	sndByte[1] = areaNum; 

	//Send command
	DspiPut(spiH, fFalse, fTrue, sndByte, NULL, 2);
	
	return fTrue;
	


}

// the following function is only available for M25P16
/* ------------------------------------------------------------------- */
/**BOOL Sleep(spiHIF * spiH) <Only available for M25P16 chip>
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**		
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function puts the device in low consumption mode (1uA)
**	
-----------------------------------------------------------------------*/
BOOL Sleep(spiHIF * spiH)
{
	int chipModel;
	int maxAdd;
	BYTE sndByte[1];

	// wait for any unfuinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);
	
	//Get Device Info	
	chipModel= GetDeviceInfo(spiH, &maxAdd);

	if(chipModel == 0x16)
	{	
		//set command  0xB9 for DEEP Power Down
		sndByte[0] = 0xB9;
	
		//Send command
		DspiPut(spiH, fFalse, fTrue, sndByte, NULL, 1);
	
		return fTrue;	
	}
	else
	{
		return fFalse;
	}


}



/* ------------------------------------------------------------------- */
/**BOOL Wake(spiHIF * spiH) <Only available for M25P16 chip>
**
**	Parameters:
**		spiH		- Pointer to spi port handle
**		
**
**	Return Value:
**		Returns 1 if successful 0 if not
**
**	Errors:
**		TBD
**
**	Description:
**		This function releases the device from low consumption mode
**
-----------------------------------------------------------------------*/
BOOL Wake(spiHIF * spiH)
{
	int chipModel;
	int maxAdd;

	BYTE sndByte[4]={0,0,0,0};
	int rcvByte[1]={0};

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	WaitForWrite(spiH);
	
	WriteEnable(spiH);
		
	//set command  0xAB to release from deep power down mode
	sndByte[0] = 0xAB;
	
	//Send command
	DspiPut(spiH, fFalse, fFalse, sndByte, NULL, 4);
	//Receive electronic signature of the device
	DspiGet(spiH, fFalse, fTrue, rcvByte, 1); 
	
	return fTrue;
	


}


